==> u++-4.7.tar.gz: u++-4.7/src/examples/FloatTest.cc <==
//                              -*- Mode: C++ -*- 
// 
// uC++ Version 4.7, Copyright (C) Peter A. Buhr and Robert Denda 1996
// 
// FloatTest.cc -- 
// 
// Author           : Peter Buhr
// Created On       : Wed May 11 17:30:51 1994
// Last Modified By : Peter A. Buhr
// Last Modified On : Sat Oct 11 16:33:25 1997
// Update Count     : 49
// 

#include <uC++.h>
#include <uIOStream.h>
#include <math.h>
#include <values.h>					// X_EPS
#ifndef X_EPS
#define X_EPS 1.0E-8
#endif


int uDefaultPreemption() {
    return 1;						// set time-slice to one millisecond
} // uDefaultPreemption


const double Range = 10000.0;


uTask Tester {
    double result;
    int TaskId;

    void main() {
	register double d;	     
	uFloatingPointContext context;			// save/restore floating point registers during a context switch

	// Each task increments a counter through a range of values and
	// calculates a trigonometric identity of each value in the
	// range. There should be numerous context switches while performing
	// this calculation. If the floating point registers are not saved
	// properly for each task, the calculations interfere producing
	// erroneous results.

	for ( d = TaskId * Range; d < (TaskId + 1) * Range; d += 1.0 ) {
	    uYield();
	    if ( fabs( sqrt( pow( sin( d ), 2.0 ) + pow( cos( d ), 2.0 ) ) - 1.0 ) > X_EPS ) { // self-check 
		uAbort( "Error during Tester[%d]:%6f", TaskId, d );
	    } // if
	    if ( d < TaskId * Range || d >= (TaskId + 1) * Range ) { // self-check 
		uAbort( "Error during Tester[%d]:%6f", TaskId, d );
	    } // if
	} // for
	result = d;
    } // Test::main
  public:
    Tester( int TaskId ) : TaskId( TaskId ) {}
    double Result() { return result; }
}; // Tester


void uMain::main() {
    const int numTesters = 10;
    Tester *testers[numTesters];
    int i;
    uFloatingPointContext context;			// save/restore floating point registers during a context switch

    for ( i = 0; i < numTesters; i += 1 ) {		// create tasks
	testers[i] = new Tester(i);
	uYield();
    } // for

    for ( i = 0; i < numTesters; i += 1 ) {		// recover results and delete tasks
	double result = testers[i]->Result();
	if ( result != (i + 1) * Range ) {		// check result
	    uAbort( "Error in Tester[%d]:%6f", i, result );
	} // if
	delete testers[i];
    } // for

    uCout << "successful completion" << endl;
} // uMain::main
    
// Local Variables: //
// compile-command: "u++ -O3 FloatTest.cc" //
// End: //

==> u++_5.0.1.orig.tar.gz: u++-5.0.1/src/examples/FloatTest.cc <==
//                              -*- Mode: C++ -*- 
// 
// uC++ Version 5.0.1, Copyright (C) Peter A. Buhr and Robert Denda 1996
// 
// FloatTest.cc -- 
// 
// Author           : Peter Buhr
// Created On       : Wed May 11 17:30:51 1994
// Last Modified By : Peter A. Buhr
// Last Modified On : Thu Aug 26 11:49:59 2004
// Update Count     : 53
// 

#include <uC++.h>
#include <uIOStream.h>
#include <cmath>
#ifndef X_EPS
#define X_EPS 1.0E-8
#endif


unsigned int uDefaultPreemption() {
    return 1;						// set time-slice to one millisecond
} // uDefaultPreemption


const double Range = 10000.0;


uTask Tester {
    double result;
    int TaskId;

    void main() {
	register double d;	     
	uFloatingPointContext context;			// save/restore floating point registers during a context switch

	// Each task increments a counter through a range of values and
	// calculates a trigonometric identity of each value in the
	// range. There should be numerous context switches while performing
	// this calculation. If the floating point registers are not saved
	// properly for each task, the calculations interfere producing
	// erroneous results.

	for ( d = TaskId * Range; d < (TaskId + 1) * Range; d += 1.0 ) {
	    uYield();
	    if ( fabs( sqrt( pow( sin( d ), 2.0 ) + pow( cos( d ), 2.0 ) ) - 1.0 ) > X_EPS ) { // self-check 
		uAbort( "Error during Tester[%d]:%6f", TaskId, d );
	    } // if
	    if ( d < TaskId * Range || d >= (TaskId + 1) * Range ) { // self-check 
		uAbort( "Error during Tester[%d]:%6f", TaskId, d );
	    } // if
	} // for
	result = d;
    } // Test::main
  public:
    Tester( int TaskId ) : TaskId( TaskId ) {}
    double Result() { return result; }
}; // Tester


void uMain::main() {
    const int numTesters = 10;
    Tester *testers[numTesters];
    int i;
    uFloatingPointContext context;			// save/restore floating point registers during a context switch

    for ( i = 0; i < numTesters; i += 1 ) {		// create tasks
	testers[i] = new Tester(i);
	uYield();
    } // for

    for ( i = 0; i < numTesters; i += 1 ) {		// recover results and delete tasks
	double result = testers[i]->Result();
	if ( result != (i + 1) * Range ) {		// check result
	    uAbort( "Error in Tester[%d]:%6f", i, result );
	} // if
	delete testers[i];
    } // for

    uCout << "successful completion" << endl;
} // uMain::main
    
// Local Variables: //
// compile-command: "u++ -O2 FloatTest.cc" //
// End: //
